/********************************************************************************
    dcttest - real-time FDCT/IDCT test
 ********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include <libvideogfx/graphics/fileio/v4l.hh>
#include <libvideogfx/x11/imgwin.hh>
#include <libvideocoding/dct/dct.hh>

#include <iostream>
using namespace std;
using namespace videocoding;

const int width=352;
const int height=288;

int main()
{
  ImageWindow_Autorefresh_X11 win(true);
  win.Create(width,height,"test");

  V4L_Grabber grabber;
  grabber.SetResolution(width,height);
  grabber.DoChromaAvgOn422To420(true);
  grabber.StartGrabbing();

  FDCTBlk8x8<Pixel>* fdct = GetFDCT8x8_8();
  IDCTBlk8x8<Pixel>* idct = GetIDCT8x8_8();

  cout << "FDCT: " << fdct->Name() << endl;
  cout << "IDCT: " << idct->Name() << endl;

  Image<Pixel> img;
  for (;;)
    {
      grabber.Grab(img);

      Pixel*const* yp = img.AskFrameY();
      int stride = yp[1]-yp[0];
      DCTBlock_8x8 blk;

      for (int y=0;y<height;y+=8)
	for (int x=0;x<width;x+=8)
	  {
	    fdct->CalcFDCT(blk, &yp[y][x],stride);

#if 0
	    blk.coeff[1][0] = 0;
	    blk.coeff[0][1] = 0;
	    blk.coeff[1][1] = 0;
#endif

	    idct->CalcIDCT(&yp[y][x],stride, blk,false);
	  }

      win.Display(img);
    }

  return 0;
}
