/********************************************************************************
    videodecoder - a complete MPEG video decoder
 ********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include <libvideocoding/fileio/inputstream_istr.hh>
#include <libvideocoding/mpeg2/syntax/pessource.hh>
#include <libvideocoding/mpeg2/syntax/demux.hh>
#include <libvideocoding/mpeg2/syntax/scsplitter.hh>
#include <libvideocoding/mpeg2/syntax/video.hh>
#include <libvideocoding/mpeg2/syntax/macroblock.hh>
#include <libvideocoding/mpeg2/decoder/video.hh>
#include <libvideocoding/dct/dct.hh>
#include <libvideogfx/utility/bitstream/bitreader.hh>
#include <libvideogfx/utility/bitstream/bitreader_fast.hh>
#include <libvideogfx/graphics/draw/draw.hh>
#include <libvideogfx/x11/imgwin.hh>

#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;
using namespace videocoding;

MP2_SequenceHeader seqhdr;

ImageWindow_Autorefresh_X11 win;


class ImageSink_My : public MP2_ImageSink
{
public:
  void SendImage(const Image<Pixel>& img,
		 const MP2_PictureHeader* first_ph,
		 const MP2_PictureHeader* second_ph,
		 const Bitmap<MP2_Macroblock>* first_pd,
		 const Bitmap<MP2_Macroblock>* second_pd)
  {
    static int cnt=1;
#if 0
    char buf[100];
    sprintf(buf,"img%05d_%d.ppm",cnt, first_ph->picture_coding_type);
    Image<Pixel> rgb;
    ChangeColorspace(rgb,img,Colorspace_RGB);
    WriteImage_PPM(rgb,buf);
    //WriteImage_JPEG(img,buf);
#endif
    printf("%d ",cnt);

#if 0
    char buf[100];
    sprintf(buf,"img%05d.jpg",cnt);
    WriteImage_JPEG(img,buf);
#endif

    switch (first_ph->picture_coding_type)
      {
      case MP2_PicType_I: printf("I\n"); break;
      case MP2_PicType_P: printf("P\n"); break;
      case MP2_PicType_B: printf("B\n"); break;
      }

    //Image<Pixel> imgg = img.Clone();

    //Clear(imgg.AskBitmapU(),(Pixel)128);
    //Clear(imgg.AskBitmapV(),(Pixel)128);

    win.Display(img);
    //usleep(2500000);
    //sleep(10);
    cnt++;

#if 0
    {
      ImageParam param = img.AskParam();
      param.width = 704;
      param.height = 576;
      Image<Pixel> cropped; cropped.Create(param);
      for (int y=0;y<param.height;y++)
	for (int x=0;x<param.width;x++)
	  {
	    cropped.AskFrameY()[y][x] = img.AskFrameY()[y][x];
	    cropped.AskFrameU()[y/2][x/2] = img.AskFrameU()[y/2][x/2];
	    cropped.AskFrameV()[y/2][x/2] = img.AskFrameV()[y/2][x/2];
	  }

      FileWriter_YUV1 writer;
      ofstream ostr("dvd.yuv");
      writer.SetYUVStream(ostr);
      writer.WriteImage(cropped);
      sleep(10);
      exit(10);
    }
#endif
  }
};


void ShowBytes(const uint8* b,int l)
{
  while (l--)
    {
      cout << hex << setfill('0') << setw(2) << ((int)b[0]) << " ";
      b++;
    }
  cout << dec << setfill(' ') << endl;
}


void ShowBits(uint8 b)
{
  int m=0x80;
  while (m)
    {
      if (b&m) cout << '1'; else cout << '0';
      m >>= 1;
    }
}

void ShowData(const uint8* b,int l)
{
  for (int i0=0;i0<l;i0+=8)
    {
      for (int i1=0;i1<8;i1++)
	{
	  if (i0+i1>=l)
	    {
	      cout << "\n";
	      return;
	    }

	  ShowBits(b[i0+i1]);
	  cout << " ";
	}
      cout << "\n";
    }

  cout << "\n";
}

int main(int argc,char** argv)
{
  try
    {
      ifstream inp(argv[1]);

      InputStream_IStream istr;
      istr.SetIStream(inp);

      //istr.SetPosition(201010000);

      MP2_UniversalStream_Decoder progdec;
      progdec.SetInputStream(istr);
      progdec.ForceStreamType(MPEG_Program_Stream);
      //progdec.ForceStreamType(MPEG_Video_Stream);

      MP2_PES_Demux demux;
      demux.SetPESSource(progdec);
      demux.EnableChannel(MP2_StartCode_Video0);

      MP2_PES_DemuxExtractor videoextr;
      videoextr.SetSource(demux);
      videoextr.SetChannel(MP2_StartCode_Video0);

      MP2_StartCodeSplitter splitter;
      splitter.SetSource(videoextr);
      splitter.SetBreakpointAllSC();
      splitter.SetBreakpoint(MP2_StartCode_ExtensionStart,false);

      MP2_PES_JunkFilter junkfilter;
      junkfilter.SetSource(splitter);

      MP2_PictureHeader pichdr;

      pichdr.seqhdr = &seqhdr;

      bool valid_seqhdr = false;
      bool valid_pichdr = false;

      ImageSink_My mysink;

      MP2_MacroblockDecoder mbdec;
      mbdec.EnableOnTheFlyDequantization(true);

      MP2_MacroblockDecompressor decoder;
      mbdec.SetMBSink(decoder);

      decoder.SetSink(mysink);

      //MP2_PictureCodingType nexttype = MP2_PicType_I;

      for (;;)
	{
	  MP2_SystemPacket* pck = junkfilter.GetNextPacket();
	  if (pck==NULL)
	    break;

	  MP2_PES_Packet* pes = dynamic_cast<MP2_PES_Packet*>(pck);
	  Assert(pes);

	  BitReader br(pes->data.AskData(), pes->data.AskLength());

	  MP2_StartCode sc = (MP2_StartCode)(br.PeekBits(32) & 0xFF);

	  switch(sc)
	    {
	    case MP2_StartCode_SequenceHeader:
	      seqhdr.Read(br);

	      if (!valid_seqhdr) // first sequence header
		{
		  win.Create(seqhdr.width, seqhdr.height, "video");
		}

	      valid_seqhdr=true;
	      mbdec.SetSequenceHeader(seqhdr);
	      decoder.SendSequenceHeader(seqhdr);
	      break;
	    case MP2_StartCode_GroupStart:
	      break;
	    case MP2_StartCode_ExtensionStart:
	      seqhdr.Read(br);
	      break;
	    case MP2_StartCode_PictureStart:
	      pichdr.Read(br);

	      valid_pichdr= (pichdr.picture_coding_type == MP2_PicType_I);
	      if (pichdr.picture_coding_type == MP2_PicType_P) valid_pichdr = true;
	      valid_pichdr = true;

	      if (valid_pichdr)
		{
		  mbdec.SetPictureHeader(pichdr);
		  decoder.SendPictureHeader(pichdr);
		}
#if 0
	      if (pichdr.picture_coding_type != nexttype)
		valid_pichdr = false;

	      if (valid_pichdr)
		{
		  if (nexttype == MP2_PicType_I)
		    nexttype = MP2_PicType_P;
		  else
		    nexttype = MP2_PicType_I;
		}
#endif

	      //cout << pichdr << endl;
	      break;
	    }

	  if (IsMPEG_SLICE_START(sc) && valid_pichdr && valid_seqhdr)
	    {
	      //ShowData(pes->data.AskData(), pes->data.AskLength());

	      MP2_SliceHeader hdr;
	      hdr.pichdr = &pichdr;
	      hdr.Read(br);

	      BitReader_Fast bsfast(&pes->data.AskData()[br.AskPosition()/8],
				    pes->data.AskLength()-br.AskPosition()/8);
	      bsfast.SkipBits(br.AskPosition()%8);

	      //ShowBytes(&pes->data.AskData()[br.Position()/8],40);
	      //cout << "skip: " << br.Position()%8 << endl;

	      mbdec.SetSliceHeader(hdr);
	      try
		{
		  mbdec.Decode(bsfast);
		}
	      catch(const Excpt_Base& e)
		{
		  MessageDisplay::Show(e);
		  //exit(10);
		}
	    }

	  delete pes;
	}

      //decoder.FlushAllPics();
    }
  catch (const Excpt_Base& e)
    {
      MessageDisplay::Show(e);
    }

  return 0;
}
