/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/datatypes/mv.hh,v 1.4 2002/05/09 22:00:31 farin Exp $

    Motion-vectors.
 ********************************************************************************
    Copyright (C) 2000 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_DATATYPES_MV_HH
#define LIBVIDEOCODING_DATATYPES_MV_HH

#include <libvideogfx/types.hh>

#include <math.h>
#include <iostream>

namespace videocoding {
  using namespace videogfx;

  typedef int32 fixed16_16; // fixed point binary, 0x00010000 = 1.0

  const fixed16_16 FullPelDist    = 0x00010000;
  const fixed16_16 HalfPelDist    = 0x00008000;
  const fixed16_16 QuarterPelDist = 0x00004000;

  /* Round towards zero to a full-pel distance. */
  inline fixed16_16 RoundTowardsZero(fixed16_16 v)
  {
    if (v<0) return (v+0xFFFF) & 0xFFFF0000;
    else     return v & 0xFFFF0000;
  }

  /* Convert to integer with rounding towards zero. */
  inline int ToInt(fixed16_16 v)
  {
    if (v<0) return (v+0xFFFF)>>16;
    else     return v>>16;
  }

  inline double     ToDouble(fixed16_16 v) { return (v>>16) + ((1.0/(double)0x10000)*(v & 0xFFFF)); }
  inline fixed16_16 IntToFixed16_16(int v) { return v<<16; }
  inline fixed16_16 DoubleToFixed16_16(double v) { return (fixed16_16)(v*0x10000); }


  struct MotionVector
  {
    MotionVector() { }
    MotionVector(fixed16_16 a_h,fixed16_16 a_v) : h(a_h), v(a_v) { }

    fixed16_16 h,v;

    bool operator==(const MotionVector& mv) const { return h==mv.h && v==mv.v; }
    bool operator!=(const MotionVector& mv) const { return h!=mv.h || v!=mv.v; }
    bool IsZero() const { return (h|v)==0; }

    MotionVector operator+(const MotionVector& mv) const
    {
      MotionVector sumMV = mv;
      sumMV.h += h;
      sumMV.v += v;
      return sumMV;
    }

    MotionVector operator*(double factor) const
    {
      MotionVector sumMV;
      sumMV.h = (fixed16_16)(h*factor);
      sumMV.v = (fixed16_16)(v*factor);
      return sumMV;
    }

    double Length() const
    {
      double dh = ToDouble(h);
      double dv = ToDouble(v);
      return sqrt(dh*dh+dv*dv);
    }

    void LimitResolution(fixed16_16 resolution) // with rounding towards smaller values
    {
      h &= ~(resolution-1);
      v &= ~(resolution-1);
    }
  };


  inline std::ostream& operator<<(std::ostream& ostr,const MotionVector& mv)
  {
    //ostr << hex << mv.h << "|" << mv.v << dec << " ";
    return ostr << ToDouble(mv.h) << '/' << ToDouble(mv.v);
  }

}

#endif
