/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/decoder/audio.hh,v 1.10 2002/08/16 17:53:56 farin Exp $

    MPEG-2 audio decoder
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_DECODER_AUDIO_HH
#define LIBVIDEOCODING_MPEG2_DECODER_AUDIO_HH

extern "C" {
#include <mad.h>
}

#include <libvideogfx/audio/fileio/audiosink.hh>
#include <libvideogfx/audio/fileio/timedsink.hh>
#include <libvideogfx/containers/queue.hh>

#include <libvideocoding/mpeg2/syntax/pessource.hh>


namespace videocoding {
  using namespace videogfx;


  class MP2_AudioDecoder
  {
  public:
    MP2_AudioDecoder();

    ~MP2_AudioDecoder();

    void SetSource(MP2_PES_Source& src) { d_src = &src; }

    void SetSink(TimedAudioSink& sink) { d_timed_sink = &sink; d_sink=&sink; }
    void SetSink(AudioSink& sink) { d_timed_sink=NULL; d_sink = &sink; }

    void Reset();

    bool DoSomething(); /* Decode some data and send it to sink. */

    // The following parameters may only be queried after some data has been decoded with DoSomething().
    int AskNChannels() const { Assert(d_params_valid); return d_channels; }
    int AskSamplerate() const { Assert(d_params_valid); return d_samplerate; }

    // --- internal use only ---

    enum mad_flow input(struct mad_stream* stream);
    enum mad_flow output(struct mad_header const *header,
			 struct mad_pcm *pcm);
    void set_params(int channels,int samplerate)
    {
      d_channels=channels;
      d_samplerate=samplerate;
      d_params_valid=true;
    }

  private:
    MP2_PES_Source* d_src;
    AudioSink*  d_sink;
    TimedAudioSink*  d_timed_sink;

    ByteBuffer d_buf;        // buffer with input data to be decoded
    uint64     d_input_pos;  // offset of d_buf[0] to beginning of stream (bytes in stream before d_buf[0])

    mad_decoder d_decoder;
    bool d_mad_initialized;

    struct PosPTS
    {
      uint64 inputpos;  // input stream position
      int64  pts;       // corresponding pts
    };

    Queue<PosPTS> d_pts_queue;
    int64 d_last_pts;
    bool  d_synchronized;
    bool  d_first_output;

    bool d_params_valid;
    int  d_channels;
    int  d_samplerate;
  };

}

#endif
