/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/encoder/quantize_mmx.hh,v 1.3 2002/05/12 21:41:47 farin Exp $

    Quantization in MMX assembler for single DCT blocks.
 ********************************************************************************
    Copyright (C) 2001 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_ENCODER_QUANTIZE_MMX_HH
#define LIBVIDEOCODING_MPEG2_ENCODER_QUANTIZE_MMX_HH

#include <libvideocoding/mpeg2/encoder/quantize.hh>

namespace videocoding {

  class MP2_Quantizer_MMX : public MP2_Quantizer
  {
  public:
    MP2_Quantizer_MMX();
    const char* AskName() const { return "MMX accelerated quantizer"; }

    void SetIntraQMatrix(const QMatrix_8x8&);
    void SetInterQMatrix(const QMatrix_8x8&);
    void SetDCPrecision(int prec);
    void SetMPEG2(bool flag)
    {
      if (flag)
	{ d_clip_low=-2048; d_clip_high=2047; }
      else
	{ d_clip_low= -256; d_clip_high= 255; }
    }

    void SetHighPrecision(bool flag=true) { d_high_precision = flag; }

    // The input and output DCT-blocks may be the same.
    void Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale);
    void Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale);

    bool Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale, int32& out_of_range);
    bool Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale, int32& out_of_range);

    void Dequantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale);
    void Dequantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale);

  private:
    bool d_high_precision;
    int  d_dc_precision;
    int  d_dc_shift;
    int  d_clip_low, d_clip_high;

    QMatrix_8x8 d_intra;
    QMatrix_8x8 d_inter;

    uint16 qmat_inter[112+1][8][8];
    uint16 qmat_intra[112+1][8][8];
    uint16 offs_intra[112+1][8][8];

    uint64 qscale_4times[112+1];
  };

}

#endif
