/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/mpeg2/syntax/macroblock.hh"
#include "libvideocoding/datatypes/dct.hh"
#include "libvideocoding/mpeg2/syntax/dct.hh"
#include "libvideocoding/mpeg2/syntax/errors.hh"
#include <libvideogfx/utility/bitstream/bitbuffer.hh>


#include <iostream>
#include <iomanip>

namespace videocoding {
  using namespace std;

#if 0
  MP2_Macroblock::MP2_Macroblock()
    : slicehdr(NULL)
  {
  }
#endif


  void MP2_Macroblock::CopyAllExceptDCTCoeff(const MP2_Macroblock& mb)
  {
    //slicehdr = mb.slicehdr;
    use_field_DCT = mb.use_field_DCT;
    //quantizer_scale_code = mb.quantizer_scale_code;  don`t copy because of ops_intra:quantize
    motion = mb.motion;
  }


  static int nr2blk[12][3] =
    {
      { 0,0,0 },
      { 0,0,1 },
      { 0,1,0 },
      { 0,1,1 },
      { 1,0,0 },
      { 2,0,0 },
      { 1,1,0 },
      { 2,1,0 },
      { 1,0,1 },
      { 1,1,1 },
      { 2,0,1 },
      { 2,1,1 }
    };

  DCTBlock_8x8&       MP2_Macroblock::AskBlock(int nr)
  {
    if (nr2blk[nr][0]==0) return dct_y [nr2blk[nr][1]][nr2blk[nr][2]];
    else if (nr2blk[nr][0]==1) return dct_cb[nr2blk[nr][1]][nr2blk[nr][2]];
    else { Assert(nr2blk[nr][0]==2);  return dct_cr[nr2blk[nr][1]][nr2blk[nr][2]]; }
  }


  const DCTBlock_8x8& MP2_Macroblock::AskBlock(int nr) const
  {
    if (nr2blk[nr][0]==0) return dct_y [nr2blk[nr][1]][nr2blk[nr][2]];
    else if (nr2blk[nr][0]==1) return dct_cb[nr2blk[nr][1]][nr2blk[nr][2]];
    else { Assert(nr2blk[nr][0]==2); return dct_cr[nr2blk[nr][1]][nr2blk[nr][2]]; }
  }


  void Debug_ShowMB(const MP2_Macroblock& mb)
  {
    cout << "qcode: " << ((int)mb.qcode) << endl;
    for (int v=0;v<16;v++)
      {
	if (v==8) cout << endl;

	for (int u=0;u<24;u++)
	  {
	    if (u==8) cout << "  ";
	    if (u==16) cout << " | ";
	    cout << setw(3);
	    if (u<8) cout << mb.dct_y[v/8][0].coeff[v%8][u];
	    else if (u<16) cout << mb.dct_y[v/8][1].coeff[v%8][u-8];
	    else
	      {
		if (v<8) cout << mb.dct_cb[0][0].coeff[v%8][u-16];
		else     cout << mb.dct_cr[0][0].coeff[v%8][u-16];
	      }
	    cout << " ";
	  }
	cout << endl;
      }
  }




  MP2_MacroblockEncoder::MP2_MacroblockEncoder()
    : d_bs(NULL),
      d_sliceconf(NULL),
      d_first_in_slice(true)
  {
  }


  struct DirectVLCTableEntry
  {
    int16 bits; int16 nBits;
  };

  struct DirectVLC
  {
    int    value;
    uint16 bits;
  };


  static DirectVLCTableEntry vlc_mbaddrinc_tab[] =
    {
      {0,0}, // Dummy-Eintrag

      /* Tabelle geklaut aus MSSG-Encoder */
      {0x01,1},  {0x03,3},  {0x02,3},  {0x03,4},
      {0x02,4},  {0x03,5},  {0x02,5},  {0x07,7},
      {0x06,7},  {0x0b,8},  {0x0a,8},  {0x09,8},
      {0x08,8},  {0x07,8},  {0x06,8},  {0x17,10},
      {0x16,10}, {0x15,10}, {0x14,10}, {0x13,10},
      {0x12,10}, {0x23,11}, {0x22,11}, {0x21,11},
      {0x20,11}, {0x1f,11}, {0x1e,11}, {0x1d,11},
      {0x1c,11}, {0x1b,11}, {0x1a,11}, {0x19,11},
      {0x18,11}, { 0, -1 }
    };

  static DirectVLC vlc_mbaddr[2048 /*2^11*/];


  static DirectVLCTableEntry vlc_mbtype_tab[4][32]=
    {
      /* table taken from MSSG encoder and slightly modified */

      { /* dummy */
	{0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}
      },

      /* I */
      {
	{0,0}, {0,0}, {0,0}, {1,1}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {0,0}, {1,2}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}
      },
      /* P */
      {
	{0,0}, {0,0}, {1,2}, {3,5}, {0,0}, {0,0}, {0,0}, {0,0},
	{1,3}, {0,0}, {1,1}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {1,6}, {1,5}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
	{0,0}, {0,0}, {2,5}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}
      },
      /* B */
      {
	{0,0}, {0,0}, {0,0}, {3,5}, {2,3}, {0,0}, {3,3}, {0,0},
	{2,4}, {0,0}, {3,4}, {0,0}, {2,2}, {0,0}, {3,2}, {0,0},
	{0,0}, {1,6}, {0,0}, {0,0}, {0,0}, {0,0}, {2,6}, {0,0},
	{0,0}, {0,0}, {3,6}, {0,0}, {0,0}, {0,0}, {2,5}, {0,0}
      }
    };


  static DirectVLCTableEntry vlc_cbp_tab[64+1]=
    {
      /* Tabelle geklaut aus MSSG-Encoder */

      {0x01,9}, {0x0b,5}, {0x09,5}, {0x0d,6}, 
      {0x0d,4}, {0x17,7}, {0x13,7}, {0x1f,8}, 
      {0x0c,4}, {0x16,7}, {0x12,7}, {0x1e,8}, 
      {0x13,5}, {0x1b,8}, {0x17,8}, {0x13,8}, 
      {0x0b,4}, {0x15,7}, {0x11,7}, {0x1d,8}, 
      {0x11,5}, {0x19,8}, {0x15,8}, {0x11,8}, 
      {0x0f,6}, {0x0f,8}, {0x0d,8}, {0x03,9}, 
      {0x0f,5}, {0x0b,8}, {0x07,8}, {0x07,9}, 
      {0x0a,4}, {0x14,7}, {0x10,7}, {0x1c,8}, 
      {0x0e,6}, {0x0e,8}, {0x0c,8}, {0x02,9}, 
      {0x10,5}, {0x18,8}, {0x14,8}, {0x10,8}, 
      {0x0e,5}, {0x0a,8}, {0x06,8}, {0x06,9}, 
      {0x12,5}, {0x1a,8}, {0x16,8}, {0x12,8}, 
      {0x0d,5}, {0x09,8}, {0x05,8}, {0x05,9}, 
      {0x0c,5}, {0x08,8}, {0x04,8}, {0x04,9},
      {0x07,3}, {0x0a,5}, {0x08,5}, {0x0c,6}, { 0, -1 }
    };


  static DirectVLCTableEntry vlc_DClum_tab[]=
    {
      /* table taken from MSSG-encoder */
      {0x0004,3}, {0x0000,2}, {0x0001,2}, {0x0005,3}, {0x0006,3}, {0x000e,4},
      {0x001e,5}, {0x003e,6}, {0x007e,7}, {0x00fe,8}, {0x01fe,9}, {0x01ff,9}, {0,-1}
    };

  static DirectVLCTableEntry vlc_DCchrom_tab[]=
    {
      /* table taken from MSSG-encoder */
      {0x0000,2}, {0x0001,2}, {0x0002,2}, {0x0006,3}, {0x000e,4}, {0x001e,5},
      {0x003e,6}, {0x007e,7}, {0x00fe,8}, {0x01fe,9}, {0x03fe,10},{0x03ff,10}, {0,-1}
    };

  static DirectVLC vlc_DClum  [1<< 9];
  static DirectVLC vlc_DCchrom[1<<10];


  void MP2_MacroblockEncoder::Write(const MP2_Macroblock& mb,int y,int x)
  {
    /*
      cout << y << " " << x << " "
      << (d_sliceconf->IsFirstInSlice(y,x) ? "F" : "-")
      << (d_sliceconf->IsLastInSlice (y,x) ? "L" : "-") << endl;
    */

    assert(d_sliceconf);
    Write(mb,d_sliceconf->IsLastInSlice(y,x));
  }


  void MP2_MacroblockEncoder::Write(const MP2_Macroblock& mb,bool last_in_slice)
  {
    MP2_SequenceHeader*   seqhdr  = mb.slicehdr->pichdr->seqhdr;
    MP2_PictureHeader*    pichdr  = mb.slicehdr->pichdr;
    MP2_PictureCodingType pictype = pichdr->picture_coding_type;

    // for convenient access to motion-vector array
    const int forward  = 0;
    const int backward = 1;

    bool is_intra = (mb.motion.prediction_mode == MP2_Intra);


    // reset predictors at beginning of slice

    if (d_first_in_slice)
      {
	d_MBincr=1;
	ResetPMV();
	ResetDCPred(pichdr);
	d_last_qcode = mb.slicehdr->qcode;
      }


    // determine coded block pattern (CBP)

    uint16 cbp = 0; // y1 y2 y3 y4  cb1 cr1 cb2 cr2  cb3 cr3 cb4 cr4

    if (pichdr->picture_coding_type == MP2_PicType_I)
      {
	// code all blocks in I-pictures

	if (seqhdr->chroma_format == MP2_Chroma_420) cbp = 0xFC0;
	else if (seqhdr->chroma_format == MP2_Chroma_422) cbp = 0xFF0;
	else cbp = 0xFFF;
      }
    else
      {
	if (!mb.dct_y [0][0].IsAllZero()) cbp |= 0x800;
	if (!mb.dct_y [0][1].IsAllZero()) cbp |= 0x400;
	if (!mb.dct_y [1][0].IsAllZero()) cbp |= 0x200;
	if (!mb.dct_y [1][1].IsAllZero()) cbp |= 0x100;

	if (!mb.dct_cb[0][0].IsAllZero()) cbp |= 0x080;
	if (!mb.dct_cr[0][0].IsAllZero()) cbp |= 0x040;

	if (seqhdr->chroma_format != MP2_Chroma_420)
	  {
	    if (!mb.dct_cb[0][1].IsAllZero()) cbp |= 0x020;
	    if (!mb.dct_cr[0][1].IsAllZero()) cbp |= 0x010;

	    if (seqhdr->chroma_format == MP2_Chroma_444)
	      {
		if (!mb.dct_cb[1][0].IsAllZero()) cbp |= 0x008;
		if (!mb.dct_cr[1][0].IsAllZero()) cbp |= 0x004;
		if (!mb.dct_cb[1][1].IsAllZero()) cbp |= 0x002;
		if (!mb.dct_cr[1][1].IsAllZero()) cbp |= 0x001;
	      }
	  }
      }


    // Test if macroblock can be skipped

    bool skip_mb=true;

    if (cbp!=0 || last_in_slice || d_first_in_slice)
      {
	skip_mb=false;
      }
    else
      {
	if (pichdr->picture_coding_type == MP2_PicType_P)
	  {
	    if (is_intra ||
		mb.motion.vector[0][forward].h != 0 || mb.motion.vector[0][forward].v != 0)
	      skip_mb=false;
	  }
	else
	  {
	    skip_mb = false;

	    if (!is_intra && !d_lastmb_was_intra &&
		mb.motion.has_mv_forward  == d_lastmb_had_mv_forward &&
		mb.motion.has_mv_backward == d_lastmb_had_mv_backward)
	      {
		if (mb.motion.has_mv_forward && mb.motion.has_mv_backward &&
		    mb.motion.vector[0][forward]  == d_PMV[0][forward]  &&
		    mb.motion.vector[0][backward] == d_PMV[0][backward])
		  {
		    skip_mb=true;
		  }
		else if (mb.motion.has_mv_forward && !mb.motion.has_mv_backward &&
			 mb.motion.vector[0][forward]  == d_PMV[0][forward])
		  {
		    skip_mb=true;
		  }
		else if (!mb.motion.has_mv_forward && mb.motion.has_mv_backward &&
			 mb.motion.vector[0][backward] == d_PMV[0][backward])
		  {
		    skip_mb=true;
		  }
	      }
	  }
      }

    if (skip_mb)
      {
	d_MBincr++;
	if (pichdr->picture_coding_type == MP2_PicType_P) ResetPMV();
	ResetDCPred(pichdr);
      }
    else
      {
	// write MB address increment

	while (d_MBincr>33)
	  { d_bs->WriteBits(8,11); d_MBincr-=33; }

	d_bs->WriteBits(vlc_mbaddrinc_tab[d_MBincr].bits,
			vlc_mbaddrinc_tab[d_MBincr].nBits);
	d_MBincr=1;


	// write MB modes

	bool mbmode_quant     = (mb.qcode != d_last_qcode);
	bool mbmode_intra     = is_intra;
	bool mbmode_pattern   = (cbp != 0); /* Differing from the MPEG standard, we define
					       mbmode_pattern to be true for I-pictures. */
	bool mbmode_mforward  = mb.motion.has_mv_forward;
	bool mbmode_mbackward = mb.motion.has_mv_backward;

	if (!mbmode_pattern) mbmode_quant=false; // If there is no image data, no quantizer change is needed.

	int mbmode=0; // Index for table lookup.

	if (mbmode_quant)     mbmode |= 0x10;
	if (mbmode_mforward)  mbmode |= 0x08;
	if (mbmode_mbackward) mbmode |= 0x04;
	if (mbmode_pattern)   mbmode |= 0x02;
	if (mbmode_intra)     mbmode |= 0x01;

	assert(mbmode>=0 && mbmode<=31);

	//cout << ((int)mb.quantizer_scale_code) << " " << d_last_qscale << endl;
	//cout << "pictype: " << ((int)pictype) << " mbmode: " << ((int)mbmode) << endl;

	assert(vlc_mbtype_tab[pictype][mbmode].bits  != 0 ||
	       vlc_mbtype_tab[pictype][mbmode].nBits != 0);

	d_bs->WriteBits(vlc_mbtype_tab[pictype][mbmode].bits,
			vlc_mbtype_tab[pictype][mbmode].nBits);

	if (mbmode_mforward || mbmode_mbackward)
	  {
	    if (pichdr->picture_structure == MP2_PicStruct_FramePicture)
	      {
		if ( ! pichdr->frame_pred_frame_dct )
		  d_bs->WriteBits(mb.motion.prediction_mode & 0xF,2);
	      }
	    else
	      {
		d_bs->WriteBits(mb.motion.prediction_mode & 0xF,2);
	      }
	  }

	if (pichdr->picture_structure == MP2_PicStruct_FramePicture &&
	    ! pichdr->frame_pred_frame_dct &&
	    (mbmode_intra || mbmode_pattern))
	  {
	    d_bs->WriteBits(mb.use_field_DCT ? 1 : 0 ,1);
	  }


	// write modified quantizer scale

	if (mbmode_quant) { d_bs->WriteBits(mb.qcode,5); }

	// write motion vectors

	if (mbmode_mforward || (mbmode_intra && pichdr->concealment_motion_vectors))
	  motion_vectors(mb,0);

	if (mbmode_mbackward)
	  motion_vectors(mb,1);

	if (mbmode_intra && pichdr->concealment_motion_vectors)
	  d_bs->WriteBits(1,1); // marker bit

	// write coded block pattern

	if (mbmode_pattern && !mbmode_intra)
	  {
	    d_bs->WriteBits(vlc_cbp_tab[cbp>>6].bits , vlc_cbp_tab[cbp>>6].nBits);

	    if      (seqhdr->chroma_format == MP2_Chroma_422) d_bs->WriteBits((cbp>>4)&3,2);
	    else if (seqhdr->chroma_format == MP2_Chroma_444) d_bs->WriteBits( cbp & 0xF,4);
	  }

	// write DCT blocks

	MP2_DCTBlockEncoder blkenc;
	blkenc.SetBitstream(*d_bs);
	blkenc.SetParams(!seqhdr->IsMPEG2,pichdr->intra_vlc_format,pichdr->alternate_scan);

	if (cbp & 0x800) blkenc.Write(mb.dct_y [0][0],d_pred_y,true,is_intra);
	if (cbp & 0x400) blkenc.Write(mb.dct_y [0][1],d_pred_y,true,is_intra);
	if (cbp & 0x200) blkenc.Write(mb.dct_y [1][0],d_pred_y,true,is_intra);
	if (cbp & 0x100) blkenc.Write(mb.dct_y [1][1],d_pred_y,true,is_intra);

	if (cbp & 0x080) blkenc.Write(mb.dct_cb[0][0],d_pred_cb,false,is_intra);
	if (cbp & 0x040) blkenc.Write(mb.dct_cr[0][0],d_pred_cr,false,is_intra);
	if (cbp & 0x020) blkenc.Write(mb.dct_cb[0][1],d_pred_cb,false,is_intra);
	if (cbp & 0x010) blkenc.Write(mb.dct_cr[0][1],d_pred_cr,false,is_intra);
								    
	if (cbp & 0x008) blkenc.Write(mb.dct_cb[1][0],d_pred_cb,false,is_intra);
	if (cbp & 0x004) blkenc.Write(mb.dct_cr[1][0],d_pred_cr,false,is_intra);
	if (cbp & 0x002) blkenc.Write(mb.dct_cb[1][1],d_pred_cb,false,is_intra);
	if (cbp & 0x001) blkenc.Write(mb.dct_cr[1][1],d_pred_cr,false,is_intra);

	if (is_intra)
	  { ResetPMV(); }
	else
	  { ResetDCPred(pichdr); }


	// prepare data for next step

	d_lastmb_had_mv_forward  = mb.motion.has_mv_forward;
	d_lastmb_had_mv_backward = mb.motion.has_mv_backward;
	d_lastmb_was_intra = is_intra;
	if (mbmode_quant) { d_last_qcode =  mb.qcode; }
      }


    if (last_in_slice)
      {
	d_first_in_slice=true; // next block is the first one of the next slice
      }
    else
      {
	d_first_in_slice=false;
      }
  }


  void MP2_MacroblockEncoder::ResetPMV()
  {
    for (int r=0;r<2;r++)
      for (int s=0;s<2;s++)
	d_PMV[r][s].h = d_PMV[r][s].v = 0;
  }


  void MP2_MacroblockEncoder::ResetDCPred(const MP2_PictureHeader* pichdr)
  {
    d_pred_y = d_pred_cb = d_pred_cr = (128<<(pichdr->intra_dc_precision-8));
  }



  void MP2_MacroblockEncoder::motion_vectors(const MP2_Macroblock& mb,int direction)
  {
    int  motion_vector_count = ((mb.motion.prediction_mode == MP2_Field_16x8MC) ? 2 : 1);
    bool mv_frame_format = (mb.motion.prediction_mode == MP2_Frame_FrameBased);

    if (motion_vector_count == 1)
      {
	if (!mv_frame_format && !mb.motion.dmv)
	  {
	    d_bs->WriteBits(mb.motion.vertical_field_select[0][direction],1);
	  }
	motion_vector(mb, 0,direction);
      }
    else
      {
	d_bs->WriteBits(mb.motion.vertical_field_select[0][direction],1);
	motion_vector(mb, 0,direction);
      
	d_bs->WriteBits(mb.motion.vertical_field_select[1][direction],1);
	motion_vector(mb, 1,direction);
      }
  }

  static struct DirectVLCTableEntry motionvectab[17]=
    {
      /* Tabelle geklaut aus MSSG-Encoder */
      {0x01,1},  {0x01,2},  {0x01,3},  {0x01,4},
      {0x03,6},  {0x05,7},  {0x04,7},  {0x03,7},
      {0x0b,9},  {0x0a,9},  {0x09,9},  {0x11,10},
      {0x10,10}, {0x0f,10}, {0x0e,10}, {0x0d,10},
      {0x0c,10}
    };

  void MP2_MacroblockEncoder::CodeMotionCode(int diff,int f,int rsize)
  {
    if (diff==0)
      d_bs->WriteBits(1,1);
    else
      {
	int absdiffS1;
	int mcode_part;
	int residual;

	absdiffS1  = (diff<0 ? -diff : diff) -1;
	mcode_part = absdiffS1 / f +1;
	residual   = absdiffS1 % f;

	d_bs->WriteBits(motionvectab[mcode_part].bits, motionvectab[mcode_part].nBits);
	d_bs->WriteBits(diff>0 ? 0 : 1 ,1);
	if (rsize) d_bs->WriteBits(residual,rsize);
      }
  }


  void MP2_MacroblockEncoder::motion_vector (const MP2_Macroblock& mb,int n,int direction)
  {
    /*
      cout << "fcode_h: " << ((int)mb.slicehdr->pichdr->f_code[direction][0])
      << "fcode_v: " << ((int)mb.slicehdr->pichdr->f_code[direction][1]) << endl;
    */    

    int rsize_h = mb.slicehdr->pichdr->f_code[direction][0]-1;
    int rsize_v = mb.slicehdr->pichdr->f_code[direction][1]-1;

    //cout << "rsize_h: " << rsize_h << "  rsize_v: " << rsize_v << endl;

    int h_f = 1<<rsize_h;
    int v_f = 1<<rsize_v;
    int hrange = 16* h_f;
    int vrange = 16* v_f;

    //cout << "hrange: " << hrange << " vrange: " << vrange << endl;

    bool fullpel = ((direction==0 && mb.slicehdr->pichdr->full_pel_forward_vector) ||
		    (direction==1 && mb.slicehdr->pichdr->full_pel_backward_vector));

    // convert sub-pel accurate motion-vector to full-pel or half-pel accuracy

    int h = mb.motion.vector[n][direction].h;
    int v = mb.motion.vector[n][direction].v;

    if (fullpel) { assert((h&0xFFFF)==0); h >>= 16; assert((v&0xFFFF)==0); v >>= 16;}
    else         { assert((h&0x7FFF)==0); h >>= 15; assert((v&0x7FFF)==0); v >>= 15;}

    //cout << "vec: " << h << "/" << v << endl;

    // VLC coding of motion-vector

    int hdiff = h - d_PMV[n][direction].h;
    if      (hdiff> hrange) hdiff -= 2*hrange;
    else if (hdiff<-hrange) hdiff += 2*hrange;

    //cout << "hdiff: " << hdiff << endl;

    CodeMotionCode(hdiff,h_f,rsize_h);

    if (mb.motion.dmv)
      { assert(0); /* TODO */ }

    int vdiff = v - d_PMV[n][direction].v;
    if      (vdiff> vrange) vdiff -= 2*vrange;
    else if (vdiff<-vrange) vdiff += 2*vrange;

    //cout << "vdiff: " << vdiff << endl;

    CodeMotionCode(vdiff,v_f,rsize_v);

    if (mb.motion.dmv)
      { assert(0); /* TODO */ }

    // set predictor

    d_PMV[n][direction].h = h;
    d_PMV[n][direction].v = v;
  }


  // ---------------------------------------- decoder ---------------------------


  MP2_MacroblockDecoder::MP2_MacroblockDecoder()
    : d_sink(NULL)
  {
    Reset();
  }

  MP2_MacroblockDecoder::~MP2_MacroblockDecoder()
  {
  }

  void MP2_MacroblockDecoder::SetMBSink(MP2_MacroblockSink& sink)
  {
    d_sink = &sink;
  }

  void MP2_MacroblockDecoder::Reset()
  {
    d_seqhdr = NULL;
    d_pichdr = NULL;
    d_slicehdr = NULL;
  }

  void MP2_MacroblockDecoder::SetSequenceHeader(const MP2_SequenceHeader& hdr)
  {
    d_seqhdr = &hdr;
  }

  void MP2_MacroblockDecoder::SetPictureHeader(const MP2_PictureHeader& hdr)
  {
    d_pichdr = &hdr;
    d_dctdecoder.SetPictureHeader(hdr);
  }

  void MP2_MacroblockDecoder::SetSliceHeader(const MP2_SliceHeader& hdr)
  {
    d_slicehdr = &hdr;
    d_qcode = hdr.qcode;

    ResetDCPredictors();
    ResetMotionPredictors();
  }

  void MP2_MacroblockDecoder::ResetDCPredictors()
  {
    d_dcpred_y = d_dcpred_cb = d_dcpred_cr = (1<<(d_pichdr->intra_dc_precision-1));
  }


  void MP2_MacroblockDecoder::ResetMotionPredictors()
  {
    d_PMV.vector[0][0].h = d_PMV.vector[0][0].v = 0;
    d_PMV.vector[0][1].h = d_PMV.vector[0][1].v = 0;
    d_PMV.vector[1][0].h = d_PMV.vector[1][0].v = 0;
    d_PMV.vector[1][1].h = d_PMV.vector[1][1].v = 0;
  }



  struct MBAtab
  {
    uint8 mba,len;
  };

  static MBAtab MBA_5 [] = {
    {7, 5}, {6, 5}, {5, 4}, {5, 4}, {4, 4}, {4, 4},
    {3, 3}, {3, 3}, {3, 3}, {3, 3}, {2, 3}, {2, 3}, {2, 3}, {2, 3},
    {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1},
    {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}, {1, 1}
  };

  static MBAtab MBA_11 [] = {
    {33, 11}, {32, 11}, {31, 11}, {30, 11},
    {29, 11}, {28, 11}, {27, 11}, {26, 11},
    {25, 11}, {24, 11}, {23, 11}, {22, 11},
    {21, 10}, {21, 10}, {20, 10}, {20, 10},
    {19, 10}, {19, 10}, {18, 10}, {18, 10},
    {17, 10}, {17, 10}, {16, 10}, {16, 10},
    {15,  8}, {15,  8}, {15,  8}, {15,  8},
    {15,  8}, {15,  8}, {15,  8}, {15,  8},
    {14,  8}, {14,  8}, {14,  8}, {14,  8},
    {14,  8}, {14,  8}, {14,  8}, {14,  8},
    {13,  8}, {13,  8}, {13,  8}, {13,  8},
    {13,  8}, {13,  8}, {13,  8}, {13,  8},
    {12,  8}, {12,  8}, {12,  8}, {12,  8},
    {12,  8}, {12,  8}, {12,  8}, {12,  8},
    {11,  8}, {11,  8}, {11,  8}, {11,  8},
    {11,  8}, {11,  8}, {11,  8}, {11,  8},
    {10,  8}, {10,  8}, {10,  8}, {10,  8},
    {10,  8}, {10,  8}, {10,  8}, {10,  8},
    { 9,  7}, { 9,  7}, { 9,  7}, { 9,  7},
    { 9,  7}, { 9,  7}, { 9,  7}, { 9,  7},
    { 9,  7}, { 9,  7}, { 9,  7}, { 9,  7},
    { 9,  7}, { 9,  7}, { 9,  7}, { 9,  7},
    { 8,  7}, { 8,  7}, { 8,  7}, { 8,  7},
    { 8,  7}, { 8,  7}, { 8,  7}, { 8,  7},
    { 8,  7}, { 8,  7}, { 8,  7}, { 8,  7},
    { 8,  7}, { 8,  7}, { 8,  7}, { 8,  7}
  };



  inline int GetMBAddrIncr(class BitReader_Fast& bs)
  {
    MBAtab * tab;
    int mba;

    mba = 0;

    bs.Fill16Bits();

    while (1) {
      if (bs.Peek16BitsMSB() >= 0x10000000) {
	tab = MBA_5 - 2 + bs.PeekBitsFast(5);
	bs.SkipBitsFast(tab->len);
	return mba + tab->mba;
      } else if (bs.Peek16BitsMSB() >= 0x03000000) {
	tab = MBA_11 - 24 + bs.PeekBitsFast(11);
	bs.SkipBitsFast(tab->len);
	return mba + tab->mba;
      } else
	{
	  switch (bs.PeekBits(11)) {
	  case 8:         // macroblock_escape
	    mba += 33;
	    // no break here on purpose
	  case 15:        // macroblock_stuffing (MPEG1 only)
	    bs.SkipBitsFast(11);
	    bs.Fill16Bits();
	    break;
	  default:        // end of slice, or error
	    throw Excpt_InvalidData("not existing MB-addr-incr. read. (1)");
	    return 0;
	  }
	}
    }
  }



  /************************************* MB-MODE ********************************/

  const uint16 MBMODE_QUANT   = 0x10;
  const uint16 MBMODE_MVFWD   = 0x08;
  const uint16 MBMODE_MVBKW   = 0x04;
  const uint16 MBMODE_PATTERN = 0x02;
  const uint16 MBMODE_INTRA   = 0x01;

  static DirectVLCTableEntry vlc_mbmode_P_tab[] =
    {
      {0,0}, {3,5}, {1,2}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
      {1,3}, {0,0}, {1,1}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
      {0,0}, {1,6}, {1,5}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0},
      {0,0}, {0,0}, {2,5}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,-1}
    };

  static DirectVLCTableEntry vlc_mbmode_B_tab[] =
    {
      {0,0}, {3,5}, {0,0}, {0,0}, {2,3}, {0,0}, {3,3}, {0,0},
      {2,4}, {0,0}, {3,4}, {0,0}, {2,2}, {0,0}, {3,2}, {0,0},
      {0,0}, {1,6}, {0,0}, {0,0}, {0,0}, {0,0}, {2,6}, {0,0},
      {0,0}, {0,0}, {3,6}, {0,0}, {0,0}, {0,0}, {2,5}, {0,0}, {0,-1}
    };


  static DirectVLC vlc_mbmode_P[64 /*2^6*/];
  static DirectVLC vlc_mbmode_B[64 /*2^6*/];


  static int GetMBMode_I(class BitReader_Fast& bs)
  {
    int code = bs.PeekBits(2);
    if (code & 2)
      {
	bs.SkipBitsFast(1);
	return 1;
      }
    else
      {
	if (code!=1)
	  throw Excpt_InvalidData("not existing MB-class (I-picture) read.");

	bs.SkipBitsFast(2);
	return 17;
      }
  }

  static int GetMBMode_P(class BitReader_Fast& bs)
  {
    uint16 code = bs.PeekBits(6);
    bs.SkipBits(vlc_mbmode_P[code].bits);

    if (vlc_mbmode_P[code].value==0)
      throw Excpt_InvalidData("not existing MB-class (P-picture) read.");

    return vlc_mbmode_P[code].value;
  }

  static int GetMBMode_B(class BitReader_Fast& bs)
  {
    uint16 code = bs.PeekBits(6);
    bs.SkipBitsFast(vlc_mbmode_B[code].bits);

    if (vlc_mbmode_B[code].value==0)
      {
	//cerr << "CODEVAL: $" << hex << code << " 6 bits " << endl;
	throw Excpt_InvalidData("not existing MB-class (B-picture) read.");
      }

    return vlc_mbmode_B[code].value;
  }

  static int GetMBMode_D(class BitReader_Fast& bs)
  {
    int c = bs.GetBits(1);
    if (c != 1)
      {
	throw Excpt_InvalidData("invalid MB-mode in D-picture read.");
      }

    return MBMODE_INTRA;
  }

  static int (* GetMBMode[5])(class BitReader_Fast&) =
  {
    NULL,
    GetMBMode_I,
    GetMBMode_P,
    GetMBMode_B,
    GetMBMode_D
  };


  static bool cbblks[12] =
    { false,false,false,false,
      true, false,true, false,
      true, false,true, false
    };


  /************************************* MotionCode ********************************/

  static DirectVLCTableEntry vlc_mcode_tab[]=
    {
      /* table taken from MSSG-encoder */

      {0x01,1},  {0x01,2},  {0x01,3},  {0x01,4},
      {0x03,6},  {0x05,7},  {0x04,7},  {0x03,7},
      {0x0b,9},  {0x0a,9},  {0x09,9},  {0x11,10},
      {0x10,10}, {0x0f,10}, {0x0e,10}, {0x0d,10},
      {0x0c,10}, {0,-1}
    };

  static DirectVLC vlc_mc[1024 /*2^10*/];

  int GetMotionCode(class BitReader_Fast& bs)
  {
    uint16 code = bs.PeekBits(10);
    bs.SkipBitsFast(vlc_mc[code].bits);

    if (vlc_mc[code].value==0)
      return 0;

    if (bs.GetBits(1)==0)
      return  vlc_mc[code].value;
    else
      return -vlc_mc[code].value;
  }


  /************************************* CBP ********************************/

  static DirectVLC vlc_cbp[512 /*2^9*/];

  /************************************* initialization ********************************/

  static void InitTAB(const DirectVLCTableEntry* src,DirectVLC* dest,int invalid_value)
  {
    int maxbits=0;
    for (uint16 n=0; src[n].nBits>=0 ;n++)
      if (src[n].nBits>maxbits)
	maxbits=src[n].nBits;

    int size = (1<<maxbits);

    for (int i=0;i<size;i++)
      dest[i].value = invalid_value;

    for (uint16 n=0; src[n].nBits>=0 ;n++)
      if (src[n].nBits>0)
	{
	  uint16 vlc_base = src[n].bits << (maxbits - src[n].nBits);
        
	  for (int i=0;i<(1<<(maxbits - src[n].nBits));i++)
	    {
	      dest[vlc_base+i].value = n;
	      dest[vlc_base+i].bits  = src[n].nBits;
	    }
	}
  }


  void InitializeVLCTabs()
  {
    static bool initialized=false;

    if (initialized)
      return;

    InitTAB(vlc_mbaddrinc_tab,vlc_mbaddr,-1);
    InitTAB(vlc_mbmode_P_tab,vlc_mbmode_P,0);
    InitTAB(vlc_mbmode_B_tab,vlc_mbmode_B,0);
    InitTAB(vlc_cbp_tab,vlc_cbp,-1);
    InitTAB(vlc_DClum_tab,vlc_DClum,-1);
    InitTAB(vlc_DCchrom_tab,vlc_DCchrom,-1);
    InitTAB(vlc_mcode_tab,vlc_mc,-1);

    initialized=true;
  }


  int GetCBP(class BitReader_Fast& bs)
  {
    uint16 code = bs.PeekBits(9);
    bs.SkipBitsFast(vlc_cbp[code].bits);
    return vlc_cbp[code].value;
  }








  void MP2_MacroblockDecoder::Decode(BitReader_Fast& bs)
  {
    InitializeVLCTabs();

    Assert(d_sink);

    int mbx = -1;
    int mby = d_slicehdr->slice_vertical_position;

    while (bs.AskBitsLeft() >= 2 && bs.PeekBits(min(bs.AskBitsLeft(),23L)) != 0)
      {
	if (!d_seqhdr->IsMPEG2)
	  {
	    // Macroblock-Stuffing

	    while (bs.PeekBits(11)==0x00F) bs.SkipBits(11);
	  }

	//cout << "Macroblock: " << hex << bs.PeekBits(32) << endl;

	int incr = GetMBAddrIncr(bs);

	// Handle skipped macroblocks. Note: the first MBincr in a slice does not define skipped MBs.
	if (incr>1 && mbx>=0)
	  {
	    for (int i=0;i<incr-1;i++)
	      {
		mbx++;

		if (mbx==d_seqhdr->AskWidthInMBs())
		  {
		    mby++; mbx=0;
		  }

		MP2_Macroblock* mb = d_sink->RequestMacroblock(mbx,mby);
		mb->slicehdr = d_slicehdr;
		mb->skipped=true;
		for (int b=0;b<12;b++)
		  mb->AskBlock(b).may_contain_data = false;

		switch (d_pichdr->picture_structure)
		  {
		  case MP2_PicStruct_FramePicture:
		    mb->motion.prediction_mode = MP2_Frame_FrameBased;
		    break;
		  case MP2_PicStruct_TopField:
		    mb->motion.prediction_mode = MP2_Field_FieldBased;
		    mb->motion.vertical_field_select[0][0] = MP2_Pred_TopField;
		    mb->motion.vertical_field_select[0][1] = MP2_Pred_TopField;
		    break;
		  case MP2_PicStruct_BottomField:
		    mb->motion.prediction_mode = MP2_Field_FieldBased;
		    mb->motion.vertical_field_select[0][0] = MP2_Pred_BottomField;
		    mb->motion.vertical_field_select[0][1] = MP2_Pred_BottomField;
		    break;
		  }

		if (d_pichdr->picture_coding_type == MP2_PicType_B)
		  {
		    mb->motion.has_mv_forward  = d_PMV.has_mv_forward;
		    mb->motion.has_mv_backward = d_PMV.has_mv_backward;

		    mb->motion.vector[0][0] = d_PMV.vector[0][0];
		    mb->motion.vector[0][1] = d_PMV.vector[0][1];

		    int shift;

		    shift = (d_pichdr->full_pel_vector[0] ? 16 : 15);
		    mb->motion.vector[0][0].h <<= shift;
		    mb->motion.vector[0][0].v <<= shift;
		    shift = (d_pichdr->full_pel_vector[1] ? 16 : 15);
		    mb->motion.vector[0][1].h <<= shift;
		    mb->motion.vector[0][1].v <<= shift;
#if 0
		    emms();

		    cout << "B skipped: ";
		    if (mb->motion.has_mv_forward)  cout << "FWD " << mb->motion.vector[0][0] << " ";
		    if (mb->motion.has_mv_backward) cout << "BKW " << mb->motion.vector[0][1] << " ";
		    cout << endl;
#endif
		  }
		else if (d_pichdr->picture_coding_type == MP2_PicType_P)
		  {
		    mb->motion.has_mv_forward  = true;
		    mb->motion.has_mv_backward = false;

		    mb->motion.vector[0][0].h = 0;
		    mb->motion.vector[0][0].v = 0;
		  }
		else
		  {
		    /* It is legal for the slice not to start in the first
		       column. But it is illegal to skip any macroblocks in I pictures. */

		    throw Excpt_InvalidData("skipped macroblock in I-picture");
		  }

		d_sink->SendMacroblock(mb,mbx,mby);
	      }

	    mbx++;
	    if (mbx==d_seqhdr->AskWidthInMBs())
	      {
		mby++; mbx=0;
	      }

	    if (d_pichdr->picture_coding_type == MP2_PicType_P)
	      ResetMotionPredictors();

	    ResetDCPredictors();
	  }
	else
	  {
	    mbx += incr;
	    while (mbx >= d_seqhdr->AskWidthInMBs())
	      {
		mbx -= d_seqhdr->AskWidthInMBs();
		mby++;
	      }
	  }

	MP2_Macroblock* mb = d_sink->RequestMacroblock(mbx,mby);
	mb->slicehdr = d_slicehdr;
	mb->skipped = false;


	// -------- macroblock_modes() ----------

	uint16 mb_mode = GetMBMode[d_pichdr->picture_coding_type](bs);

	/* TODO: Spatial_Temporal_Weight_Code missing */

	if (mb_mode & (MBMODE_MVFWD|MBMODE_MVBKW))
	  {
	    if (d_pichdr->picture_structure == MP2_PicStruct_FramePicture)
	      {
		if (d_pichdr->frame_pred_frame_dct)
		  mb->motion.prediction_mode = MP2_Frame_FrameBased;
		else
		  mb->motion.prediction_mode = (MP2_Prediction_Mode)bs.GetBits(2);
	      }
	    else
	      {
		mb->motion.prediction_mode = (MP2_Prediction_Mode)(bs.GetBits(2)+0x10);
	      }
	  }
	else if ((mb_mode & MBMODE_INTRA) && d_pichdr->concealment_motion_vectors)
	  {
	    if (d_pichdr->picture_structure == MP2_PicStruct_FramePicture)
	      mb->motion.prediction_mode = MP2_Frame_FrameBased;
	    else
	      mb->motion.prediction_mode = MP2_Field_FieldBased;
	  }

	mb->use_field_DCT=false;
	if (d_seqhdr->IsMPEG2 &&
	    d_pichdr->picture_structure == MP2_PicStruct_FramePicture &&
	    d_pichdr->frame_pred_frame_dct == false &&
	    (mb_mode & (MBMODE_INTRA|MBMODE_PATTERN)))
	  {
	    mb->use_field_DCT = bs.GetBits(1);
	  }

	// -------- end of macroblock_modes() -----------

	if (!(mb_mode & MBMODE_INTRA))
	  ResetDCPredictors();

	// Reset motion-vector predictions
	if (((mb_mode & MBMODE_INTRA) && !d_pichdr->concealment_motion_vectors) ||
	    (d_pichdr->picture_coding_type == MP2_PicType_P &&
	     !(mb_mode & MBMODE_MVFWD) && !(mb_mode & MBMODE_INTRA)))
	  {
	    ResetMotionPredictors();
	  }

	if (mb_mode & MBMODE_QUANT)
	  d_qcode = bs.GetBits(5);

	mb->qcode = d_qcode;
	d_dctdecoder.SetQCode(d_qcode);


	// ------------- motion vectors -----------------

	int motion_vector_count = ((mb->motion.prediction_mode == MP2_Frame_FieldBased ||
				    mb->motion.prediction_mode == MP2_Field_16x8MC) ? 2 : 1);

	if ((mb_mode & MBMODE_MVFWD) ||
	    ((mb_mode & MBMODE_INTRA) && d_pichdr->concealment_motion_vectors))
	  {
	    motion_vectors(mb->motion,0,bs);

	    if (motion_vector_count==1) d_PMV.vector[1][0] = d_PMV.vector[0][0];    // Nach Table 7-9
	  }

	if (mb_mode & MBMODE_MVBKW)
	  {
	    motion_vectors(mb->motion,1,bs);

	    if (motion_vector_count==1) d_PMV.vector[1][1] = d_PMV.vector[0][1];    // Nach Table 7-9
	  }

#if 0
	if (1)
	  {
	    cout << "PMV[0][0][]: " << d_PMV[0][0].h << "," << d_PMV[0][0].v << endl;
	    cout << "PMV[0][1][]: " << d_PMV[0][1].h << "," << d_PMV[0][1].v << endl;
	    cout << "PMV[1][0][]: " << d_PMV[1][0].h << "," << d_PMV[1][0].v << endl;
	    cout << "PMV[1][1][]: " << d_PMV[1][1].h << "," << d_PMV[1][1].v << endl;
	  }
#endif

	if ((mb_mode & MBMODE_INTRA) && d_pichdr->concealment_motion_vectors)
	  {
	    bs.SkipBits(1);
	  }

	if (d_pichdr->picture_coding_type == MP2_PicType_P && !(mb_mode & (MBMODE_MVFWD|MBMODE_INTRA)))
	  {
	    mb_mode |= MBMODE_MVFWD;
	    mb->motion.vector[0][0].h = 0;
	    mb->motion.vector[0][0].v = 0;
	    if (d_pichdr->picture_structure==MP2_PicStruct_FramePicture)
	      {
		mb->motion.prediction_mode = MP2_Frame_FrameBased;
	      }
	    else if (d_pichdr->picture_structure == MP2_PicStruct_TopField)
	      {
		mb->motion.prediction_mode = MP2_Field_FieldBased;
		mb->motion.vertical_field_select[0][0] = MP2_Pred_TopField;
	      }
	    else
	      {
		mb->motion.prediction_mode = MP2_Field_FieldBased;
		mb->motion.vertical_field_select[0][0] = MP2_Pred_BottomField;
	      }
	  }

	if (mb_mode & MBMODE_INTRA) mb->motion.prediction_mode = MP2_Intra;

	mb->motion.has_mv_forward  = (mb_mode & MBMODE_MVFWD);
	mb->motion.has_mv_backward = (mb_mode & MBMODE_MVBKW);

	d_PMV.has_mv_forward  = mb->motion.has_mv_forward;  // for prediction of skipped MBs in  B-pics
	d_PMV.has_mv_backward = mb->motion.has_mv_backward; // ""

	// ------- begin of coded_block_pattern() -----------

	uint16 cbp = 0;

	if (mb_mode & MBMODE_PATTERN)
	  {
	    cbp=GetCBP(bs)<<6;

	    if (d_seqhdr->chroma_format==MP2_Chroma_422)
	      { cbp |= bs.GetBits(2)<<4; }
	    else if (d_seqhdr->chroma_format==MP2_Chroma_444)
	      { cbp |= bs.GetBits(6);    }
	  }


	// ------- end of coded_block_pattern() -----------



	int dctblks;
	switch (d_seqhdr->chroma_format)
	  {
	  case MP2_Chroma_420: dctblks =  6; break;
	  case MP2_Chroma_422: dctblks =  8; break;
	  case MP2_Chroma_444: dctblks = 12; break;
	  }

	d_dctdecoder.SetQCode(d_qcode);

	if (mb_mode & MBMODE_INTRA)
	  {
	    for (int b=0;b<dctblks;b++)
	      {
		DCTBlock_8x8& blk = mb->AskBlock(b);
		blk.Clear();
		blk.may_contain_data = true;

		int* dcpredictor;

		if (b<4)            { dcpredictor = &d_dcpred_y;  }
		else if (cbblks[b]) { dcpredictor = &d_dcpred_cb; }
		else                { dcpredictor = &d_dcpred_cr; }

		blk.coeff[0][0] = *dcpredictor;
		*dcpredictor = d_dctdecoder.DecodeIntra(blk, bs, (b<4));
	      }
	  }
	else
	  {
	    for (int b=0;b<dctblks;b++)
	      {
		DCTBlock_8x8& blk = mb->AskBlock(b);

		if (!(cbp & (0x800>>b)))
		  {
		    blk.may_contain_data = false;
		    continue;
		  }

		blk.Clear();
		blk.may_contain_data = true;
		d_dctdecoder.DecodeInter(blk, bs, (b<4));
	      }
	  }



	d_sink->SendMacroblock(mb,mbx,mby);
      }
  }


  void MP2_MacroblockDecoder::motion_vectors(struct MP2_Motion& mv,int s, class BitReader_Fast& bs)
  {
    int motion_vector_count = ((mv.prediction_mode == MP2_Frame_FieldBased ||
				mv.prediction_mode == MP2_Field_16x8MC) ? 2 : 1);

    bool FieldMVInFramePic = (mv.prediction_mode == MP2_Frame_FieldBased ||
			      mv.prediction_mode == MP2_Frame_DualPrime);

    int shift = (d_pichdr->full_pel_vector[s] ? 16 : 15);

    if (motion_vector_count == 1)
      {
	if (mv.prediction_mode == MP2_Field_FieldBased)
	  { mv.vertical_field_select[0][s] = (MP2_PredictionField)bs.GetBits(1); }

	motion_vector(mv,0,s,bs);

	d_PMV.vector[0][s] = mv.vector[0][s];
	mv.vector[0][s].h <<= shift;
	mv.vector[0][s].v <<= shift;

	if (FieldMVInFramePic) d_PMV.vector[0][s].v *= 2;
      }
    else
      {
	mv.vertical_field_select[0][s] = (MP2_PredictionField)bs.GetBits(1);
	motion_vector(mv,0,s,bs);
	mv.vertical_field_select[1][s] = (MP2_PredictionField)bs.GetBits(1);
	motion_vector(mv,1,s,bs);

	d_PMV.vector[0][s] = mv.vector[0][s];
	mv.vector[0][s].h <<= shift;
	mv.vector[0][s].v <<= shift;

	d_PMV.vector[1][s] = mv.vector[1][s];
	mv.vector[1][s].h <<= shift;
	mv.vector[1][s].v <<= shift;

	if (FieldMVInFramePic) d_PMV.vector[0][s].v *= 2;
	if (FieldMVInFramePic) d_PMV.vector[1][s].v *= 2;
      }
  }

  inline void DIV2(fixed16_16& x) { if (x>=0) x/=2; else x=(x-1)/2; }

  void MP2_MacroblockDecoder::motion_vector(struct MP2_Motion& mv,int r, int s, class BitReader_Fast& bs)
  {
    /*
      cout << "prediction[" << r << "][" << s << "] = "
      << d_PMV.vector[r][s].h << ","
      << d_PMV.vector[r][s].v << endl;
    */

    bool FieldMVInFramePic = (mv.prediction_mode == MP2_Frame_FieldBased ||
			      mv.prediction_mode == MP2_Frame_DualPrime);

    bool dmv = (mv.prediction_mode == MP2_Frame_DualPrime ||
		mv.prediction_mode == MP2_Field_DualPrime);

    // horizontal component

    {
      const int motcode = GetMotionCode(bs);
      const int fcode   = d_pichdr->f_code[s][0];
      const int rsize   = fcode-1;
      int delta;
      int res;

      if (motcode!=0 && fcode !=1)
	{
	  res = bs.GetBits(rsize);
	  if (motcode>0)
	    delta =   (( motcode-1)<<rsize) + res+1;
	  else
	    delta = -(((-motcode-1)<<rsize) + res+1);
	}
      else
	delta = motcode;

      //cout << "delta: " << delta << ",";
    
      mv.vector[r][s].h = d_PMV.vector[r][s].h + delta;
    
      if      (mv.vector[r][s].h < (-16 << rsize))
	mv.vector[r][s].h += 32<<rsize;
      else if (mv.vector[r][s].h > ( 16 << rsize)-1)
	mv.vector[r][s].h -= 32<<rsize;
    }

    if (dmv)
      {
	int bits = bs.PeekBits(2);
	switch (bits)
	  {
	  case 0:
	  case 1: mv.dmvector[r][s][0]= 0; bs.SkipBits(1); break;
	  case 2: mv.dmvector[r][s][0]= 1; bs.SkipBits(2); break;
	  case 3: mv.dmvector[r][s][0]=-1; bs.SkipBits(2); break;
	  }
      }



    // vertical component

    {
      const int motcode = GetMotionCode(bs);
      const int fcode   = d_pichdr->f_code[s][1];
      const int rsize   = fcode-1;
      int delta;
      int res;

      if (motcode!=0 && fcode !=1)
	{
	  res = bs.GetBits(rsize);
	  if (motcode>0)
	    delta =   (( motcode-1)<<rsize) + res+1;
	  else
	    delta = -(((-motcode-1)<<rsize) + res+1);
	}
      else
	delta = motcode;

      //cout << delta << endl;
    
      mv.vector[r][s].v = d_PMV.vector[r][s].v;
      if (FieldMVInFramePic) DIV2(mv.vector[r][s].v);

      mv.vector[r][s].v += delta;
    
      if      (mv.vector[r][s].v < (-16 << rsize))
	mv.vector[r][s].v += 32<<rsize;
      else if (mv.vector[r][s].v > ( 16 << rsize)-1)
	mv.vector[r][s].v -= 32<<rsize;
    }

    if (dmv)
      {
	int bits = bs.PeekBits(2);
	switch (bits)
	  {
	  case 0:
	  case 1: mv.dmvector[r][s][0]= 0; bs.SkipBits(1); break;
	  case 2: mv.dmvector[r][s][0]= 1; bs.SkipBits(2); break;
	  case 3: mv.dmvector[r][s][0]=-1; bs.SkipBits(2); break;
	  }
      }
  }

}
