/********************************************************************************
    Copyright (C) 2002  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#include "libvideocoding/mpeg2/syntax/tsinput.hh"
#include "libvideocoding/mpeg2/syntax/errors.hh"
#include <libvideogfx/error.hh>
#include <algorithm>
#include <string.h>

namespace videocoding {
  using namespace videogfx;

  InputStream_TS::InputStream_TS()
    : d_istr(NULL),
      d_packet_valid(false),
      d_idx(0)
  {
  }


  InputStream_TS::~InputStream_TS()
  {
  }


  uint32 InputStream_TS::MyFillBuffer(uint8* mem,uint32 maxlen)
  {
    Assert(d_istr);

    if (!ReadNextPacket())
      return false;

    Assert(d_packet_valid);

    int cpylen = std::min((int32)maxlen, (int32)(d_current.payload_length-d_idx));
    memcpy(mem, &d_current.payload[d_idx], cpylen);
    d_idx += cpylen;

    return cpylen;
  }


  bool InputStream_TS::ReadNextPacket()
  {
    Assert(d_istr);

    while (!d_packet_valid ||                     // no current TS-packet
	   d_idx == d_current.payload_length)     // all of current TS-packet has been read
      {
	try
	  {
	    d_current.Read(*d_istr);
	    d_packet_valid = true;
	    d_idx = 0;
	  }
	catch (const Excpt_UnexpectedEndOfData&)
	  {
	    return false;
	  }
	catch (...)
	  {
	    // try again
	  }
      }

    return true;
  }


  bool InputStream_TS::IsEOF() const
  {
    Assert(d_istr);

    if (BytesInPushbackQueue()) return false;

    if (d_packet_valid && d_idx<d_current.payload_length) return false;

    if (const_cast<InputStream_TS*>(this)->ReadNextPacket())
      return false;

    return true;
  }

}
